#include "stdafx.h"

#include "NPRTFDrawer.h"
/////////////////////////////////////////////////////////////////////////////
// CallBack functions

DWORD CALLBACK EditStreamInCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
{
	PCOOKIE pCookie = (PCOOKIE) dwCookie;

	if(pCookie->dwSize - pCookie->dwCount < (DWORD) cb)
		*pcb = pCookie->dwSize - pCookie->dwCount;
	else
		*pcb = cb;

	CopyMemory(pbBuff, pCookie->bstrText, *pcb);
	pCookie->dwCount += *pcb;

	return 0;	//	callback succeeded - no errors
}

/////////////////////////////////////////////////////////////////////////////
// CNPRTFDrawer

HRESULT CNPRTFDrawer::get_Text(BSTR *pVal)
{
	*pVal = SysAllocStringLen(m_RTFText, SysStringLen(m_RTFText));
	return S_OK;
}

HRESULT CNPRTFDrawer::put_Text(BSTR newVal)
{
	HRESULT hr;
	long    len;
	LRESULT lResult = 0;
	EDITSTREAM editStream;

	if (m_RTFText != NULL)
		SysFreeString(m_RTFText);
	len = SysStringLen(newVal);
	m_RTFText = SysAllocStringLen(newVal, len);

	if (!m_spTextServices) 
		return S_FALSE;

	m_editCookie.bstrText = (BSTR) malloc(len + 1);
	WideCharToMultiByte(CP_ACP, 0, m_RTFText, len, (char *) m_editCookie.bstrText, len, NULL, NULL);
	m_editCookie.dwSize = lstrlenA((LPSTR) m_editCookie.bstrText);
	m_editCookie.dwCount = 0;

	editStream.dwCookie = (DWORD_PTR) &m_editCookie;
	editStream.dwError = 0;
	editStream.pfnCallback = EditStreamInCallback;
	//hr = m_spTextServices->TxSendMessage(EM_STREAMIN, (WPARAM)(SF_TEXT | SF_UNICODE), (LPARAM)&editStream, &lResult);
	hr = m_spTextServices->TxSetText(newVal);
	return S_OK;
}
HRESULT CNPRTFDrawer::put_CharFormat(CHARFORMAT& cf)
{
	if (!m_spTextServices) 
		return S_FALSE;
	LRESULT lResult = 0;
	HRESULT hr = m_spTextServices->TxSendMessage(EM_SETCHARFORMAT, WPARAM(SCF_ALL), LPARAM(&cf), &lResult);
	return hr;
}
HRESULT CNPRTFDrawer::put_ParaFormat(PARAFORMAT& pf)
{
	if (!m_spTextServices) 
		return S_FALSE;
	LRESULT lResult = 0;
	PARAFORMAT2 pf2;
	ZeroMemory(&pf2, sizeof(pf2));
	pf2.cbSize = sizeof(pf2);
	pf2.dwMask = PFM_ALL2;
	pf2.wAlignment = pf.wAlignment;
	pf2.bLineSpacingRule = 0;
	HRESULT hr = m_spTextServices->TxSendMessage(EM_SETPARAFORMAT, WPARAM(0), LPARAM(&pf2), &lResult);
	return hr;
}
HRESULT CNPRTFDrawer::Draw(void *hdcDraw, RECT *prc)
{
	if (!m_spTextServices) 
		return S_FALSE;
	m_spTextServices->TxDraw(
		DVASPECT_CONTENT,  		// Draw Aspect
		0,						// Lindex
		NULL,					// Info for drawing optimization
		NULL,					// target device information
		(HDC) hdcDraw,				// Draw device HDC
		NULL,			 	   	// Target device HDC
		(RECTL *) prc,			// Bounding client rectangle
		NULL,					// Clipping rectangle for metafiles
		(RECT *) NULL,			// Update rectangle
		NULL, 	   				// Call back function
		NULL,					// Call back parameter
		TXTVIEW_INACTIVE);		// What view of the object could be TXTVIEW_ACTIVE
	return S_OK;
}

HRESULT CNPRTFDrawer::Create()
{
	return CreateTextServicesObject();
}

HRESULT CNPRTFDrawer::get_NaturalWidth(long Height, long *pVal)
{
	long lWidth;
	SIZEL szExtent;
	HDC	hdcDraw;

	if (!m_spTextServices)
		return S_FALSE;

	hdcDraw = GetDC(NULL);
	szExtent.cy = Height;
	szExtent.cx = 10000;
	lWidth = 10000;
	m_spTextServices->TxGetNaturalSize(DVASPECT_CONTENT, 
		hdcDraw, 
		NULL,
		NULL,
		TXTNS_FITTOCONTENT,
		&szExtent,
		&lWidth,
		&Height);

	ReleaseDC(NULL, hdcDraw);
	*pVal = lWidth;
	return S_OK;
}

HRESULT CNPRTFDrawer::get_NaturalHeight(long Width, long *pVal)
{
	long lHeight;
	SIZEL szExtent;
	HDC	hdcDraw;

	if (!m_spTextServices)
		return S_FALSE;

	hdcDraw = GetDC(NULL);
	szExtent.cx = Width;
	szExtent.cy = 10000;
	lHeight = 10000;
	m_spTextServices->TxGetNaturalSize(DVASPECT_CONTENT, 
		hdcDraw, 
		NULL,
		NULL,
		TXTNS_FITTOCONTENT,
		&szExtent,
		&Width,
		&lHeight);

	ReleaseDC(NULL, hdcDraw);
	*pVal = lHeight;
	return S_OK;
}
LRESULT CNPRTFDrawer::SendMessage(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if (!m_spTextServices) 
		return 0;
	LRESULT lResult = 0;
	HRESULT hr = m_spTextServices->TxSendMessage(uMsg, wParam, lParam, &lResult);
	return lResult;
}
/////////////////////////////////////////////////////////////////////////////
// ITextHost functions
HDC CNPRTFDrawer::TxGetDC()
{
	return NULL;
}

INT CNPRTFDrawer::TxReleaseDC(HDC hdc)
{
	return 1;
}

BOOL CNPRTFDrawer::TxShowScrollBar(INT fnBar, BOOL fShow)
{
	return FALSE;
}

BOOL CNPRTFDrawer::TxEnableScrollBar(INT fuSBFlags, INT fuArrowflags)
{
	return FALSE;
}

BOOL CNPRTFDrawer::TxSetScrollRange(INT fnBar, LONG nMinPos, INT nMaxPos, BOOL fRedraw)
{
	return FALSE;
}

BOOL CNPRTFDrawer::TxSetScrollPos(INT fnBar, INT nPos, BOOL fRedraw)
{
	return FALSE;
}

void CNPRTFDrawer::TxInvalidateRect(LPCRECT prc, BOOL fMode)
{
}

void CNPRTFDrawer::TxViewChange(BOOL fUpdate)
{
}

BOOL CNPRTFDrawer::TxCreateCaret(HBITMAP hbmp, INT xWidth, INT yHeight)
{
	return FALSE;
}

BOOL CNPRTFDrawer::TxShowCaret(BOOL fShow)
{
	return FALSE;
}

BOOL CNPRTFDrawer::TxSetCaretPos(INT x, INT y)
{
	return FALSE;
}

BOOL CNPRTFDrawer::TxSetTimer(UINT idTimer, UINT uTimeout)
{
	return FALSE;
}

void CNPRTFDrawer::TxKillTimer(UINT idTimer)
{
}

void CNPRTFDrawer::TxScrollWindowEx(INT dx, INT dy, LPCRECT lprcScroll, LPCRECT lprcClip, HRGN hrgnUpdate, LPRECT lprcUpdate, UINT fuScroll)
{
}

void CNPRTFDrawer::TxSetCapture(BOOL fCapture)
{
}

void CNPRTFDrawer::TxSetFocus()
{
}

void CNPRTFDrawer::TxSetCursor(HCURSOR hcur, BOOL fText)
{
}

BOOL CNPRTFDrawer::TxScreenToClient(LPPOINT lppt)
{
	return FALSE;
}

BOOL CNPRTFDrawer::TxClientToScreen(LPPOINT lppt)
{
	return FALSE;
}

HRESULT	CNPRTFDrawer::TxActivate(LONG * plOldState)
{
	return S_OK;
}

HRESULT	CNPRTFDrawer::TxDeactivate(LONG lNewState)
{
	return S_OK;
}

HRESULT	CNPRTFDrawer::TxGetClientRect(LPRECT prc)
{
	*prc = m_rcClient;
	return S_OK;
}

HRESULT	CNPRTFDrawer::TxGetViewInset(LPRECT prc)
{
	*prc = m_rcViewInset;
	return S_OK;
}

HRESULT CNPRTFDrawer::TxGetCharFormat(const CHARFORMATW **ppCF)
{
	//*ppCF = m_pCF;
	return S_FALSE;
}

HRESULT	CNPRTFDrawer::TxGetParaFormat(const PARAFORMAT **ppPF)
{
	//*ppPF = &m_PF;
	return S_FALSE;
}

COLORREF CNPRTFDrawer::TxGetSysColor(int nIndex)
{
	return GetSysColor(nIndex);
}

HRESULT	CNPRTFDrawer::TxGetBackStyle(TXTBACKSTYLE *pstyle)
{
	*pstyle = TXTBACK_TRANSPARENT;
	return S_OK;
}

HRESULT	CNPRTFDrawer::TxGetMaxLength(DWORD *plength)
{
	*plength = m_dwMaxLength;
	return S_OK;
}

HRESULT	CNPRTFDrawer::TxGetScrollBars(DWORD *pdwScrollBar)
{
	*pdwScrollBar = m_dwScrollbar;
	return S_OK;
}

HRESULT	CNPRTFDrawer::TxGetPasswordChar(TCHAR *pch)
{
	return S_FALSE;
}

HRESULT	CNPRTFDrawer::TxGetAcceleratorPos(LONG *pcp)
{
	*pcp = -1;
	return S_OK;
}

HRESULT	CNPRTFDrawer::TxGetExtent(LPSIZEL lpExtent)
{
	return E_NOTIMPL;
}

HRESULT CNPRTFDrawer::OnTxCharFormatChange(const CHARFORMATW * pcf)
{
	//memcpy(m_pCF, pcf, pcf->cbSize);
	return S_OK;
}

HRESULT	CNPRTFDrawer::OnTxParaFormatChange(const PARAFORMAT * ppf)
{
	//memcpy(&m_PF, ppf, ppf->cbSize);
	return S_OK;
}

HRESULT	CNPRTFDrawer::TxGetPropertyBits(DWORD dwMask, DWORD *pdwBits)
{
	*pdwBits = m_dwPropertyBits;
	return S_OK;
}

HRESULT	CNPRTFDrawer::TxNotify(DWORD iNotify, void *pv)
{
	return S_OK;
}

HIMC CNPRTFDrawer::TxImmGetContext()
{
	return NULL;
}

void CNPRTFDrawer::TxImmReleaseContext(HIMC himc)
{
}

HRESULT	CNPRTFDrawer::TxGetSelectionBarWidth(LONG *lSelBarWidth)
{
	*lSelBarWidth = 100;
	return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
// custom functions
HRESULT CNPRTFDrawer::CreateTextServicesObject()
{
	HRESULT hr;
	IUnknown *spUnk;

	hr = CreateTextServices(NULL, static_cast<ITextHost*>(this), &spUnk);
	if (hr == S_OK) {
		hr = spUnk->QueryInterface(D_IID_ITextServices, (void**)&m_spTextServices);
		hr = spUnk->QueryInterface(IID_ITextDocument, (void**)&m_spTextDocument);
		spUnk->Release();
	}
	if (m_spTextServices)
	{
		LRESULT lResult = 0;
		SendMessage(EM_SETTEXTMODE, WPARAM(TM_PLAINTEXT), LPARAM(0));
		SendMessage(EM_SETWORDWRAPMODE , WPARAM(WBF_WORDWRAP), LPARAM(0));
	}
	return hr;
}

